@@ -8,7 +8,7 @@ def test_config(api):
     config.a = 'asdf'
     config.b = 1.1
     config.c = 1
-    config.d = [config.A, config.B, config.C]
+    config.d_values = [config.A, config.B, config.C]
     config.e.e_a = 1.1
     config.e.e_b = 1.2
     config.f.f_a = 'a'
